--------------------------------------------------------------------------
--  DLX PROCESSOR MODEL SUITE
--  Copyright (C) 1995, Martin Gumm
--  University of Stuttgart / Department of Computer Science / IPVR-ISE
--------------------------------------------------------------------------
--  This program is free software; you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation; either version 1, or (at your option)
--  any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--------------------------------------------------------------------------
--  Last revision date : November 15 1995
--------------------------------------------------------------------------

--------------------------------------------------------------------------
--  *** SYNOPSYS synthesizable code (ver. 3.2.a) ***
--------------------------------------------------------------------------

--------------------------------------------------------------------------
--  Structural architecture of the core of the DLX processor
--  (only signed instructions)
--  
--  (file core-structural.vhd)
--------------------------------------------------------------------------

architecture structural of core2 is

  component data_siding
  port (addr_pc  : in dlx_word;
        addr_mar : in dlx_word;
        data_lmdr : out dlx_word;
        data_smdr : in dlx_word;
        data_ir   : out dlx_word;
        d_bus_input : in dlx_word;
        d_bus_output : out dlx_word;
        a_bus_output : out dlx_word;
        ready_in     : in std_logic;
        ready_data    : out std_logic;
        ready_instr   : out std_logic;
        --
        mem_ctrl_in : in std_logic_vector(0 to 7);
        mem_ctrl_out : out std_logic_vector(0 to 7);
        fetch_mem_ctrl : in std_logic);
  end component;  

  component controller2
  port (phi1, phi2  : in std_logic;                  -- two phase clock in
	reset : in std_logic;                       -- reset input
	halt  : in std_logic;                       -- halt input
	ready_data : in std_logic;                       -- mem. ready input
        ready_instr : in std_logic;
        --
	instr_in : in dlx_word;                     -- instruction input
	alu_zero : in std_logic;                    -- alu zero bit
	alu_neg  : in std_logic;                    -- alu negative bit
	alu_ovfl : in std_logic;                    -- alu overflow bit
	intrpt   : in std_logic;                    -- interrupt active
	super    : in std_logic;                    -- supervisor bit 
        --
	-- reg. address outputs
	--
        --
	-- control outputs
	--
        ir_latch_en_out : out std_logic;
        fetch_mem_ctrl : out std_logic;
	--
	--
        a_b_latch_out : out std_logic;
        t_and_s : out std_logic;
        test_result : out std_logic;
        ir_1_latch : out std_logic;
	rs1_out : out dlx_reg_addr;
	rs2_out : out dlx_reg_addr;	  
        reg_zero : in std_logic;
        reg_zero_sel : out std_logic_vector(0 to 1);
        --
	--
        alu_mux_1_sel : out std_logic_vector(0 to 1);   -- forwarded results
        alu_mux_2_sel : out std_logic_vector(0 to 1);   
        smdr_mux_sel  : out std_logic_vector(0 to 1);
        s1_enab    : out std_logic_vector(0 to 5);    -- select s1 source
        s2_enab    : out std_logic_vector(0 to 3);    -- select s2_source
        alu_op_sel : out std_logic_vector(0 to 3);    -- alu operation
        dest_enab  : out std_logic_vector(0 to 4);    -- select destination
        const_sel  : out std_logic_vector(0 to 1);    -- select const for s1
        immed_sel  : out std_logic_vector(0 to 1);    -- select immed. from ir
        exc_enab   : out std_logic_vector(0 to 8);    -- enable set exc. bit
        icr_shifter : out std_logic;
        iar_mux : out std_logic;
        iar_latch : out std_logic;
        smdr_latch : out std_logic;
        smdr_ctrl : out std_logic_vector(0 to 2);
	--
	--
	mem_ctrl  : out std_logic_vector(0 to 7);   -- memory control lines
	entry_mux_sel : out std_logic;
        c_latch     : out std_logic;
        pc_latch    : out std_logic;
        pc_mux_sel  : out std_logic;
        lmdr_ctrl : out std_logic_vector(0 to 2);
        lmdr_latch  : out std_logic;
	--
	--
	wb_mux_sel  : out std_logic;
	rd_out      : out dlx_reg_addr;
        rf_in_out   : out std_logic;
	--
	--
        adr_ls2_in     : in std_logic_vector(1 downto 0);
        adr_msb_in     : in std_logic);

  end component;

  component datapath2
  port(signal data_in        : in  dlx_word;           -- data in from pads
       signal instr_in       : in dlx_word;            -- instruction in
       signal data_out       : out dlx_word;           -- data_out to pads
       signal data_addr_out  : out dlx_word;           -- data-addr. out to pads
       signal instr_addr_out : out dlx_word;           -- instr-addr.
       signal instr_out      : out dlx_word;           -- instr. reg. content
       signal alu_zero       : out std_logic;                     
       signal alu_negative   : out std_logic;
       signal alu_overflow   : out std_logic;
       signal icr_intrpt     : out std_logic;          -- at least one masked intrpt.
       signal icr_super    : out std_logic;          -- supervisor bit in icr

       signal mar_adr_ls2  : out std_logic_vector(0 to 1); -- least 2 addr. bits
       signal mar_adr_msb  : out std_logic;          --  msb of mar
       --
       -- reg. addr. inputs
       --
       signal reg_addr_rs1 : in dlx_reg_addr;
       signal reg_addr_rs2 : in dlx_reg_addr;
       signal reg_addr_rd  : in dlx_reg_addr;
       --
       -- control inputs
       --
       signal a_latch_en  	: in std_logic;         -- low active
       signal a_out_en    	: in std_logic;         -- low active
       signal b_latch_en  	: in std_logic;         -- low active
       signal b_out_en    	: in std_logic;         -- low active
       signal ir_1_latch_en     : in std_logic;
       signal c_latch_en  	: in std_logic;         -- low active
       signal aluout_latch_en   : in std_logic;
       signal regf_wr_en  	: in std_logic;         -- low active

       signal reg_zero          : out std_logic;
       signal reg_zero_sel      : in std_logic_vector(0 to 1);

       signal mar_latch_en : in std_logic;        -- low active

       signal lmdr_latch_en : in std_logic;        -- low active
       signal lmdr_ctrl     : in std_logic_vector(0 to 2);
       signal smdr_latch_en : in std_logic;

       signal smdr_ctrl  : in std_logic_vector(0 to 2); -- enab signal for shift op.


       signal pc_latch_en  : in std_logic;        -- low active
       signal pc_out_en    : in std_logic;        -- low active

       signal iar_latch_en : in std_logic;        -- low active
       signal iar_out_en   : in std_logic;        -- low active
       signal tbr_latch_en : in std_logic;        -- low active
       signal tbr_out_en   : in std_logic;        -- low active

       signal icr_out_en   : in std_logic;        -- low active
       signal icr_latch_en : in std_logic;        -- low active
       signal icr_s_en     : in std_logic;        -- enab set s bit (low act.)
       signal icr_ioc_en   : in std_logic;        -- enab set ioc bit (low act.)
       signal icr_irra_en  : in std_logic;        -- enab set irra bit(low act.)
       signal icr_iav_en   : in std_logic;        -- enab set iav bit (low act.)
       signal icr_dav_en   : in std_logic;        -- enab set dav bit (low act.)
       signal icr_ovad_en   : in std_logic;       -- enab set ovad bit(low act.)
       signal icr_ovar_en   : in std_logic;       -- enab set ovar bit(low act.)
       signal icr_priv_en   : in std_logic;       -- enab set priv bit (low act.)
       signal icr_intrpt_in : in dlx_nibble;      -- intrpt. input from pads
       signal icr_intrpt_en : in std_logic;       -- enable setting of intrpt bits 

       signal icr_reset : in std_logic;
       
       signal ir_latch_en    : in std_logic;      -- low active
       signal ir_immed_o1_en : in std_logic;      -- low active
       signal ir_immed_o2_en : in std_logic;      -- low active
       signal ir_immed_size  : in std_logic;      -- '0'-> 16 bit / '1'-> 26 bit
       signal ir_immed_sign  : in std_logic;      -- '0'-> unsign./ '1'-> sign.  

       signal alu_latch_en : in std_logic;        -- low active
       signal alu_func     : in dlx_nibble;

       signal const_o1_en : in std_logic;         -- enable constant for s1 (low act.)
       signal const_o2_en : in std_logic;         -- enable constant for s2 (low act.)
       signal const_sel   : in std_logic_vector(0 to 1);  -- select constant

       signal alu_mux_1_sel : in std_logic_vector(0 to 1);
       signal alu_mux_2_sel : in std_logic_vector(0 to 1);
       signal smdr_mux_sel : in std_logic_vector(0 to 1);

       signal wb_mux_sel  : in std_logic;
       signal entry_mux_sel : in std_logic;
       signal t_and_s       : in std_logic;
       signal test_result   : in std_logic;
       signal iar_mux_sel   : in std_logic;
       signal pc_mux_sel    : in std_logic);
              
end component;
  --
  -- signals to connect
  --
  signal neg_phi1_in   : std_logic;

  signal dp_instr_out  : dlx_word; 
  signal dp_alu_zero   : std_logic;                     
  signal dp_alu_neg    : std_logic;
  signal dp_alu_ovfl   : std_logic;
  signal dp_intrpt_out : std_logic;
  signal dp_super_out  : std_logic;
  signal dp_adr_msb_out : std_logic;
  signal dp_adr_ls2_out : std_logic_vector(1 downto 0);
  signal dp_reg_zero   : std_logic;
  --
  signal c_ir_latch_en : std_logic;
  --
  signal c_rs1_out    : dlx_reg_addr;
  signal c_rs2_out    : dlx_reg_addr;
  signal c_a_b_latch_out : std_logic;  
  signal c_ir_1_latch : std_logic;
  --
  signal c_t_and_s : std_logic;
  signal c_test_result : std_logic;
  signal c_reg_zero_sel : std_logic_vector(0 to 1);
  --
  signal c_alu_mux_1_sel : std_logic_vector(0 to 1);
  signal c_alu_mux_2_sel : std_logic_vector(0 to 1);
  signal c_smdr_mux_sel : std_logic_vector(0 to 1);

  signal c_s1_enab    : std_logic_vector(0 to 5);    -- select s1 source
  signal c_s2_enab    : std_logic_vector(0 to 3);    -- select s2_source
  signal c_dest_enab  : std_logic_vector(0 to 4);    -- select destination
  signal c_alu_op_sel : std_logic_vector(0 to 3);    -- alu operation
  signal c_const_sel  : std_logic_vector(0 to 1);
  signal c_immed_sel  : std_logic_vector(0 to 1);    -- select immediate from ir
  signal c_exc_enab   : std_logic_vector(0 to 8);    -- enable set exception bt
  signal c_icr_shifter : std_logic;
  signal c_iar_mux_sel : std_logic;
  signal c_iar_latch  : std_logic;
  signal c_smdr_latch : std_logic;
  signal c_smdr_ctrl  : std_logic_vector(0 to 2);
  --
  signal c_lmdr_ctrl   : std_logic_vector(0 to 2);
  signal c_mem_ctrl      : std_logic_vector(0 to 7);   -- memory control lines
  signal c_entry_mux_out : std_logic;
  signal c_c_latch       : std_logic;
  signal c_pc_latch      : std_logic;
  signal c_pc_mux_sel    : std_logic;
  signal c_lmdr_latch    : std_logic;
  --
  signal c_rd_out     : dlx_reg_addr;
  signal c_rf_in      : std_logic;
  signal c_wb_mux_sel : std_logic;
  --
  --
  signal c_fetch_mem_ctrl : std_logic;
  --
  --
  --
  -- signals for and from the data_siding unit
  --
  signal instr_addr : dlx_word;
  signal data_addr  : dlx_word;
  signal input_data : dlx_word;
  signal output_data : dlx_word;
  signal instr_word  : dlx_word;

  signal ds_mem_ctrl : std_logic_vector(0 to 7);
  signal ds_ready_instr : std_logic;
  signal ds_ready_data  : std_logic;

begin       

  the_switch : data_siding
    port map (addr_pc => instr_addr,
              addr_mar => data_addr,
              data_lmdr => input_data,
              data_smdr => output_data,
              data_ir => instr_word,
              d_bus_input => d_bus_in,
              d_bus_output => d_bus_out,
              a_bus_output => a_bus,
              ready_in => ready,
              ready_data => ds_ready_data,
              ready_instr => ds_ready_instr,
              mem_ctrl_in => c_mem_ctrl,
              mem_ctrl_out => ds_mem_ctrl,
              fetch_mem_ctrl => c_fetch_mem_ctrl);


  the_controller : controller2
    port map (phi1 => phi1,
   	      phi2 => phi2,
 	      halt => halt,
 	      reset => reset,
	      ready_data => ds_ready_data,
              ready_instr => ds_ready_instr,
	      --
	      --
 	      instr_in => dp_instr_out,
              alu_zero => dp_alu_zero,
	      alu_neg => dp_alu_neg,
 	      alu_ovfl => dp_alu_ovfl,
 	      intrpt => dp_intrpt_out,
	      super => dp_super_out,
	      --
	      --
	      ir_latch_en_out => c_ir_latch_en,
              fetch_mem_ctrl => c_fetch_mem_ctrl,
	      --
	      --
	      ir_1_latch => c_ir_1_latch,
              a_b_latch_out => c_a_b_latch_out,
              t_and_s => c_t_and_s,
              test_result => c_test_result,
	      rs1_out => c_rs1_out,
	      rs2_out => c_rs2_out,
              reg_zero => dp_reg_zero,
              reg_zero_sel => c_reg_zero_sel,
	      --
	      --
              alu_mux_1_sel => c_alu_mux_1_sel,
              alu_mux_2_sel => c_alu_mux_2_sel,
              smdr_mux_sel => c_smdr_mux_sel,

	      s1_enab => c_s1_enab,
 	      s2_enab => c_s2_enab,
  	      dest_enab => c_dest_enab,
 	      alu_op_sel => c_alu_op_sel,
	      const_sel => c_const_sel,
	      immed_sel => c_immed_sel,
 	      exc_enab => c_exc_enab,
              icr_shifter => c_icr_shifter,
              iar_mux => c_iar_mux_sel,
              iar_latch => c_iar_latch,
              smdr_latch => c_smdr_latch,
              smdr_ctrl => c_smdr_ctrl,
	      --
	      --
	      mem_ctrl => c_mem_ctrl,
	      entry_mux_sel => c_entry_mux_out,
              c_latch => c_c_latch,
              pc_latch => c_pc_latch,
              pc_mux_sel => c_pc_mux_sel,
              lmdr_ctrl => c_lmdr_ctrl,
              lmdr_latch => c_lmdr_latch,
	      --
	      --
	      wb_mux_sel => c_wb_mux_sel,
	      rf_in_out => c_rf_in,
	      rd_out => c_rd_out,
	      --
	      --
              adr_ls2_in => dp_adr_ls2_out,
	      adr_msb_in => dp_adr_msb_out);
	      
  the_datapath : datapath2
    port map (data_in => input_data,
	      data_out => output_data,
	      data_addr_out => data_addr,
              instr_in => instr_word,
              instr_addr_out => instr_addr,
	      --
	      --
	      instr_out => dp_instr_out,
	      alu_zero => dp_alu_zero,
	      alu_negative => dp_alu_neg,
	      alu_overflow => dp_alu_ovfl,
	      icr_intrpt => dp_intrpt_out,
	      icr_super => dp_super_out,
	      mar_adr_msb => dp_adr_msb_out,
	      mar_adr_ls2 => dp_adr_ls2_out,
	      --
              ir_latch_en => c_ir_latch_en,
	      --
	      --
	      -- input from decode actions
	      --
	      reg_addr_rs1 => c_rs1_out,
	      reg_addr_rs2 => c_rs2_out,
              a_latch_en => c_a_b_latch_out,
              b_latch_en => c_a_b_latch_out,
              t_and_s => c_t_and_s,
              test_result => c_test_result,              
              ir_1_latch_en => c_ir_1_latch,
              
	      --
	      reg_zero => dp_reg_zero,
              reg_zero_sel => c_reg_zero_sel,
              --

	      -- s1_enab(0 to 5) : select source for s1 bus
	      -- |a|icr|iar|tbr|mdr|immed|consta| (b/pc/mar not connected to s1 bus)
	      --
	      a_out_en => c_s1_enab(0),
	      icr_out_en => c_s1_enab(1),
	      iar_out_en => c_s1_enab(2),
	      tbr_out_en => c_s1_enab(3),
	      ir_immed_o1_en => c_s1_enab(4),
	      const_o1_en => c_s1_enab(5),
	      --
	      -- s2_enab(0 to 3) : select source for s2 bus
	      -- |b|immed|pc|mar|constb|  (a/mdr/icr/tbr/iar not connected to s2 bus)
	      --
	      b_out_en => c_s2_enab(0),
	      ir_immed_o2_en => c_s2_enab(1),  
	      pc_out_en => c_s2_enab(2),
	      const_o2_en => c_s2_enab(3),       
	      --	      
	      -- dest_enab(0 to 6) : select destination from dest bus
	      -- |c|icr|iar|tbr|pc|mar|mdr|
	      --
              alu_mux_1_sel => c_alu_mux_1_sel,
              alu_mux_2_sel => c_alu_mux_2_sel,
              smdr_mux_sel => c_smdr_mux_sel,
	      aluout_latch_en => c_dest_enab(0),
	      icr_latch_en => c_dest_enab(1),
	      iar_latch_en => c_dest_enab(2),
	      tbr_latch_en => c_dest_enab(3),
	      mar_latch_en => c_dest_enab(4),
	      --
	      --
	      -- alu_op_sel(0 to 3) : select alu operation
	      --
	      alu_func => c_alu_op_sel,
              alu_latch_en => neg_phi1_in,
	      --
	      -- const_sel(0 to 2) (select constant for s2 bus)
  	      --
	      const_sel => c_const_sel,
	      --
	      -- immed_sel(0 to 1) : select immediate type 
	      -- |size_imm|sign_imm|
	      --
 	      ir_immed_size => c_immed_sel(0),
	      ir_immed_sign => c_immed_sel(1),
	      --
	      --
	      -- exc_enab(0 to 8) : switch input / enable setting of exception bits
	      -- |s|ioc|irra|iav|dav|ovad|ovar|priv|reset|
	      --
	      icr_s_en => c_exc_enab(0),
	      icr_ioc_en => c_exc_enab(1),
	      icr_irra_en => c_exc_enab(2),
	      icr_iav_en => c_exc_enab(3),
	      icr_dav_en => c_exc_enab(4),
	      icr_ovad_en => c_exc_enab(5),
	      icr_ovar_en => c_exc_enab(6),
	      icr_priv_en => c_exc_enab(7),
              icr_reset => c_exc_enab(8),
	      icr_intrpt_in => intrpt,
	      icr_intrpt_en => phi2,
	      --
              smdr_latch_en => c_smdr_latch,
              smdr_ctrl => c_smdr_ctrl, 
	      --
              lmdr_ctrl => c_lmdr_ctrl,
              lmdr_latch_en => c_lmdr_latch,
              c_latch_en => c_c_latch,
              pc_latch_en => c_pc_latch,
              pc_mux_sel => c_pc_mux_sel,
	      --
	      -- mem_ctrl(0 to 7) : all control signal for memory interface etc.
	      -- |enable(0 to 3)|rw|pad_d_inout|pad_out_en|error|
	      --
              --
              --
              --
	      --
	      entry_mux_sel => c_entry_mux_out,
 	      iar_mux_sel => c_iar_mux_sel,
	      --
	      --
              reg_addr_rd => c_rd_out,
	      wb_mux_sel => c_wb_mux_sel,
              regf_wr_en => c_rf_in);



  --
  -- negate phi1 signal
  --
  neg_phi1_in <= not phi1;

  --
  -- connect pad control lines
  --

  -- enable <= "0000" when (phi2'event and phi2 = '0') else 
  -- ds_mem_ctrl(0 to 3) when (phi1'event and phi1 = '1') else
  -- ds_mem_ctrl(0 to 3);

  enable <= ds_mem_ctrl(0 to 3);
  rw <= ds_mem_ctrl(4);
  pad_io_sw <= ds_mem_ctrl(5);
  pad_out_en <= ds_mem_ctrl(6);
  error <= ds_mem_ctrl(7);

  



end structural;

